import os


class BaseEngine(object):
    
    database = ''

    def __init__(self, database_name="indexdir"):
        self.database_name = database_name
        database_folder_path = "/tmp/epub_worker/databases/"

        if not os.path.exists(database_folder_path):
            os.mkdir(database_folder_path)

        self.database_path = database_folder_path + database_name
        self.open()
        pass

    def open(self):
        '''
        Opens the database for reading
        '''
        pass

    def create(self):
        '''
        Creates the database if it is not present
        Otherwise open current DB

        Setup writing to DB
        '''
        pass

    def clear(self):
        '''
        Clears the current database if it present
        '''
        pass

    def add(self, path='', href='', title='', cfiBase='', spinePos=''): 
        '''
        Called to load a single document from the spine
            - path = Relative path to the chapter
            - href = URL to chapter from the manifest
            - title = Title from the TOC
            - cfiBase = Chapter Base of the EPUBCFI
            - spinePos = position in the book (starting at 1) 
        '''
        pass

    def finished(self):
        '''
        Cleanup after adding all the chapters
        '''
        pass

    def query(self, q, limit=None):
        '''
        Returns a List of results containing Dicts with the following keys: 
            title
            href
            path
            title
            cfiBase
            spinePos
        '''
        results = []
        
        hit = [] # replace with DB query

        for hit in hits:
            item = {}
            item['title']   = hit["title"]
            item['href']    = hit["href"]
            item['path']    = hit["path"]
            item['title']   = hit["title"]
            item['cfiBase'] = hit["cfiBase"]
            item['spinePos']= hit["spinePos"]

            results.append(item)

        return results



import zipfile
import shutil
import os

import logging
import logging.handlers

from optparse import OptionParser

try:
    from .epubsearch import EpubParser
    from .epubsearch import EpubIndexer
    from .epubsearch import WordMorphoGenerator
except:
    from epubsearch import EpubParser
    from epubsearch import EpubIndexer
    from epubsearch import WordMorphoGenerator

syslog = logging.handlers.SysLogHandler(address = '/dev/log')
logging.basicConfig(format="%(levelname)s:%(asctime)s %(message)s",  level=logging.DEBUG,
                    handlers=[syslog])


def unzip(source_filename, dest_dir):
    with zipfile.ZipFile(source_filename) as zf:
        path = dest_dir
        zf.extractall(path)


class EpubWorker(object):
    """
    This module is main class. For using as library needed import this class.
    @:book_address - address for dir or compressed epub publication
        :parameter "./test_data/Sensei4.epub"
    @:lang - need for generating lexemes
        :parameter "ru" or "en"
    @:force_index - if you need force reindex book
        :parameter True/False
    """
    def __init__(self, book_address, lang='ru', force_index=False):
        force_index=force_index
        self.is_epub = False
        epub_worker_folder = "/tmp/epub_worker"
        if not os.path.exists(epub_worker_folder):
            os.mkdir(epub_worker_folder)

        if book_address[-4:] == 'epub':
            self.is_epub = True
            logging.info('Uncompress {}'.format(book_address))
            book_name = book_address[book_address.rfind('/')+1:-5]
            self.dest_dir = '/tmp/epub_worker/temp/'+book_name
            unzip(book_address, self.dest_dir)
            book_address = self.dest_dir
        else:
            book_name = book_address[book_address.rfind('/')+1:]

        self.epub = EpubParser(book_address)
        self.index = EpubIndexer(engine_name='whoosh', database_name=book_name, force_index=force_index)
        logging.info('Indexing')
        self.index.load(self.epub)

    def search_word(self, search_word):
        logging.info('Search word {}'.format(search_word))
        return self.index.search(search_word)

    def search_lexemes(self, search_word):
        logging.info('Generate words for search')
        search_words = WordMorphoGenerator(search_word).generate()
        logging.info('Search word {} and lexemes'.format(search_word, search_words))
        results_dirty = []
        results_formatted = []
        for word in search_words:
            results_dirty.append(self.index.search(word))
        for result in results_dirty:
            result = result.get('results')
            if result:
                for item in result:
                    results_formatted.append({'baseCfi': item['baseCfi'],
                                              'cfi': item['cfi'],
                                              'href': item['href'],
                                              'path': item['path']})

        return {'word': search_word,
                'lexemes': search_words,
                'results': results_formatted}

    def get_chapters_cfi(self):
        spine = self.epub.spine
        characters_cfi = [row['cfiBase'] for row in spine]
        return characters_cfi

    def close(self):
        shutil.rmtree(self.dest_dir)

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        if self.is_epub:
            shutil.rmtree(self.dest_dir)



def get_parameters():
    """
        Parse the user input
    """
    parser = OptionParser()
    parser.add_option('-b', '--book-address', dest='book_address')
    parser.add_option('-s', '--search', dest='search')
    parser.add_option('-f', '--force-index', dest='force_index')
    parser.add_option('--lang', dest='language')
    parser.add_option('--lexemes', dest='lexemes')
    (options, args) = parser.parse_args()

    if not options.book_address:
        options.book_address = "Sensei4/"
    else:
        return {'book_address': options.book_address,
                'search': options.search, 'language': options.language,
                'lexemes': options.lexemes, 'force_index': options.force_index}


def main():
    logging.info('*'*20)
    # get user defined parameters
    userParams = get_parameters()

    search = userParams['search']
    book_address = userParams['book_address']
    language = userParams['language']
    lexemes = userParams['lexemes']
    force_index = userParams['force_index']

    with EpubWorker(book_address, language, force_index) as worker:
        if lexemes:
            return worker.search_lexemes(search)
        return worker.search_word(search)

if __name__ == '__main__':
    # logging.basicConfig(format='%(levelname)s:%(asctime)s %(message)s', level=logging.DEBUG)
    print(main())




try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='epubsearcher',
    version='0.1.4',
    packages=['epubsearcher', 'epubsearcher.epubsearch', 'epubsearcher.epubsearch.morpho_engines',
              'epubsearcher.epubsearch.search_engines'],
    url='https://bitbucket.org/Infernion/epubsearcher/overview',
    license='MIT',
    author='Sergiy Khalymon',
    author_email='sergiykhalimon@gmail.com',
    description='Search selected word in epub book.',
    install_requires = [
        "whoosh",
        "pymorphy2",
        "lxml",
        "BeautifulSoup4"
    ],
)



# -*- coding: utf-8 -*-
import importlib
import logging

class WordMorphoGenerator(object):
    engine = False

    def __init__(self, word, lang='ru'):
        if lang=='ru': engine_name='pymorphy2'
        if engine_name:
            self.word = word

            mod = importlib.import_module("epubsearcher.epubsearch.morpho_engines.%sengine" % engine_name)
            # import whooshengine as engine
            self.engine = getattr(mod,'%sEngine' % engine_name.capitalize())
            logging.info(self.engine)

    def generate(self):
        '''
        run morpho procces for selected engine
        :return: list of words
        '''
        result = self.engine(self.word).process()
        return result


class BaseEngine(object):

    def __init__(self, word):
        self.word = word

    def parse(self):
        '''
        parse word with specefic engine
        '''
        assert NotImplemented

    def parse_normal(self):
        '''
        parse word in normal form
        '''
        assert NotImplemented

    def normal_form(self):
        '''
        generate normal word form
        '''
        assert  NotImplemented

    def lexeme(self):
        '''
        return lexemes of word
        '''
        assert NotImplemented

    def process(self):
        self.parse()
        self.normal_form()
        self.parse_normal()
        self.lexeme()
        if self.lexemes:
            return self.lexemes
        else:
            raise Exception
